package rocks.zipcode;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

/**
 * @author leon on 16/11/2018.
 */
public class Document implements DocumentInterface {

    private final FileWriter fileWriter;
    private final File file;

    public Document(String fileName) throws IOException {
        this(new File(fileName));
    }

    public Document(File file) throws IOException {
        this.file = file;
        this.fileWriter = new FileWriter(file);
    }

    @Override
    public void write(String contentToBeWritten) {

        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(file));
            bw.write(contentToBeWritten);
            bw.close();
        } catch (IOException e) {
            System.out.println("Error writing to file");
        }
    }

    @Override
    public void write(Integer lineNumber, String valueToBeWritten) {

            String line = read(lineNumber);
            String doc = read();
            doc = doc.replace(line, valueToBeWritten);
            write(doc);

    }

    @Override
    public String read(Integer lineNumber) {

        BufferedReader reader = null;
        String line = null;

        try {
            reader = new BufferedReader(new FileReader(file));
            for (int i = 0; i <= lineNumber; i++) {

                line = reader.readLine();
            }

        } catch (FileNotFoundException e) {
            System.out.println("File not found");
        } catch (IOException ioExep) {
            ioExep.printStackTrace();
        } finally {
            try {
                reader.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return line;
    }

    @Override
    public String read() {

        BufferedReader reader = null;
        String line = null;
        StringBuilder stringBuilder = new StringBuilder();
        String ls = System.getProperty("line.separator");

        try {
            reader = new BufferedReader(new FileReader(file));
            while ((line = reader.readLine()) != null) {
                stringBuilder.append(line);
                stringBuilder.append(ls);
            }

        } catch (FileNotFoundException e) {
            System.out.println("File not found");
        } catch (IOException ioExep) {
            ioExep.printStackTrace();
        } finally {
            try {
                reader.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return stringBuilder.toString().trim();
    }

    @Override
    public void replaceAll(String stringToReplace, String replacementString) {

        String doc = read();
        doc = doc.replaceAll(stringToReplace, replacementString);
        write(doc);

    }

    @Override
    public void overWrite(String content) {
        write(content);
    }

    public List<String> toList() {
        String[] arr = read().split("\n");
        List<String> result = new ArrayList<>();
        for(String s: arr) {
            result.add(s);
        }
        return result;
    }

    @Override
    public File getFile() {
        return this.file;
    }

    @Override
    public String toString() {
        return getFile() + "{" + read() + "}";
    }
}
